# -*- coding: iso_8859_1 -*-

from aspylib import astro, license, conf, photometry
import numpy as np
import time, os, datetime, warnings
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
from matplotlib.widgets import Button
from matplotlib import cm
plt.ion()

#------------------------------------------------------------------------------------
#	Purpose 
#------------------------------------------------------------------------------------
#
#	Opens the text file created by the script "photosearch_run.py" and allows the user to display all lightcurves one by one, and look for variable stars.
#	For more explanations see: http://www.aspylib.com/doc/using_cdr.html
#
#------------------------------------------------------------------------------------
#		Input parameters
#------------------------------------------------------------------------------------

#--- input frames ---
Folder = u"C:\\images\\"

File = Folder + "target_03_magnitudes.txt"
Image = Folder + 'image-calibrated-1.fit'

mode = 1
#mode = 1 --> plot is magnitude / global noise
#mode = 2 --> plot is local noise / global noise

#------------------------------------------------------------------------------------
#		Script
#------------------------------------------------------------------------------------
#*********************************************************************************************************
#		Reading information
#*********************************************************************************************************

print
data = astro.load_data(File,string_start="#--- JULIAN DAY",string_end="\n")
print "number of images:",np.shape(data)[0]
print "measured objects:",np.shape(data)[1]-1
print

obj = astro.load_data(File,string_start="#--- OBJECTS",string_end="\n",colstart=1)
astrom = astro.load_data(File,string_start="#--- ASTROMETRY",string_end="\n",colstart=1)
usnob1 = astro.load_data(File,string_start="#--- MATCHING",string_end="\n",colstart=1)
stats = astro.load_data(File,string_start="#--- STATISTICS",string_end="\n",colstart=1)

print
print "nb of objects with given quality:"
print "quality 0:",np.sum(stats[3,:]==0)
print "quality 1:",np.sum(stats[3,:]==1)
print "quality 2:",np.sum(stats[3,:]==2)
print "quality 3:",np.sum(stats[3,:]==3)
print "quality 4:",np.sum(stats[3,:]==4)
print

ind0 = list(np.where(np.array(stats[3,:])==0)[0])
ind1 = list(np.where(np.array(stats[3,:])==1)[0])
ind2 = list(np.where(np.array(stats[3,:])==2)[0])
ind3 = list(np.where(np.array(stats[3,:])==3)[0])
ind4 = list(np.where(np.array(stats[3,:])==4)[0])

flagplot = stats[3,:] <= 4

#--- calculating magnitude constant ---
refstars = astro.load_data(File,string_start="#--- REFERENCE",string_end="\n",colstart=1)
refstars = np.array([refstars[2], refstars[5], refstars[8]], dtype=np.int)

if len(usnob1)>0:		#to check that information about astrometric reduction is there
	mag = []
	ref = []
	for i in refstars[0]:
		try:
			mag.append(float(usnob1[6,i]))
			ref.append(i)
		except:
			pass

	ind = [i+1 for i in ref]
	magoffset = np.mean(mag) - np.mean(data[:,ind],axis=1)
	meanmagoffset = np.mean(magoffset[np.logical_not(np.isnan(magoffset))])
else:
	magoffset = np.array(0.*data[:,0])
	meanmagoffset = 0.
	
#*********************************************************************************************************
#		Plotting stats information
#*********************************************************************************************************

xy = []

fig = plt.figure(figsize=(10,10))
fig.subplots_adjust(bottom=0.08)
fig.subplots_adjust(top=0.95)
ax = plt.subplot(211)
if mode==1:
	if len(ind0)>0:
		ax.scatter(stats[0,ind0]+meanmagoffset,stats[1,ind0],color='g', s=10, label='< 0.05')
	if len(ind1)>0:
		ax.scatter(stats[0,ind1]+meanmagoffset,stats[1,ind1],color='#bbfe00', s=10, label='< 0.10')
	if len(ind2)>0:
		ax.scatter(stats[0,ind2]+meanmagoffset,stats[1,ind2],color='y', s=10, label='< 0.25')
	if len(ind3)>0:
		ax.scatter(stats[0,ind3]+meanmagoffset,stats[1,ind3],color='#fe8c00', s=10, label='< 0.50')
	if len(ind4)>0:
		ax.scatter(stats[0,ind4]+meanmagoffset,stats[1,ind4],color='r', s=10, label='> 0.50')
	plt.xlabel('instrumental magnitude')
	plt.ylabel('global magnitude noise')
elif mode==2:
	if len(ind0)>0:
		ax.scatter(stats[2,ind0],stats[1,ind0],color='g', s=10, label='< 0.05')
	if len(ind1)>0:
		ax.scatter(stats[2,ind1],stats[1,ind1],color='#bbfe00', s=10, label='< 0.10')
	if len(ind2)>0:
		ax.scatter(stats[2,ind2],stats[1,ind2],color='y', s=10, label='< 0.25')
	if len(ind3)>0:
		ax.scatter(stats[2,ind3],stats[1,ind3],color='#fe8c00', s=10, label='< 0.50')
	if len(ind4)>0:
		ax.scatter(stats[2,ind4],stats[1,ind4],color='r', s=10, label='> 0.50')
	plt.xlabel('local magnitude noise')
	plt.ylabel('global magnitude noise')
plt.grid()
plt.title('Select star with a right-click')
box = ax.get_position()
ax.set_position([box.x0, box.y0, box.width * 0.75, box.height])
leg = plt.legend(title = r'$\mathrm{\mathsf{max(|m_{i+1}-m_i|)}}$', prop=fm.FontProperties(size=11), loc='center left', bbox_to_anchor=(1.02, 0.79))
leg.get_title().set_fontproperties(fm.FontProperties(size=13))

#*********************************************************************************************************
#		Plotting lightcurve and star image
#*********************************************************************************************************

def export_cdrcdl(event):
	f = open(Folder+"object_"+str(index+1)+"_cdrcdl.txt", 'w')
	# f.write("-------------------------------------------------\n")
	# f.write("      Object "+str(index+1)+"\n")
	# f.write("-------------------------------------------------\n\n")
	f.write(strinfo)
	f.write("\n\n")
	f.write("Column 1: geocentric Julian date at mid-exposure\n")
	f.write("Column 2: differential magnitude, differs from true magnitude by a constant offset\n")
	f.write("Column 3: magnitude noise\n")
	f.write("------------------------------------------------------------------------\n")
	f.write("FMT DVv\n")
	f.write("NOM star_"+str(index+1)+"\n")
	if usnob1[1,index]=='-':
		f.write(";alpha="+(astrom[0,index]).replace(":"," ")+", delta="+(astrom[1,index]).replace(":"," ")+"\n")
	else:
		f.write(";USNO-B1 "+usnob1[1,index]+" alpha="+(usnob1[2,index]).replace(":"," ")+", delta="+(usnob1[3,index]).replace(":"," ")+"\n")
	f.write("MES\n")
	for i in range(len(data)):
		f.write("%(JJ).6f\t%(mag).6f\t%(sigma).6f\n" % {'JJ':data[i,0],'mag':data[i,index+1]+magoffset[i],'sigma':stats[2,index]})
	f.close()
	print "object "+str(index+1)+" exported to .txt file"

warnings.simplefilter("ignore") 
ax1 = fig.add_subplot(223)
ax2 = fig.add_subplot(224)
box = ax1.get_position()
ax1.set_position([box.x0*0.8, box.y0, box.width * 0.7, box.height])
box = ax2.get_position()
ax2.set_position([box.x0*0.8, box.y0, box.width * 1.4, box.height])
but1 = Button(plt.axes([0.78,0.6,0.15,0.075]),'Export to\nCdR-CdL file')
but1.on_clicked(export_cdrcdl)

try:
	image = astro.get_imagedata([Image],text_display=False)
	image = np.array(image,dtype=np.int)
except:
	image = np.zeros((1,1))

flag = False
index_old = None

while 1:
	def on_click(event):
		global ax1, ax2, index, strinfo, flag, index_old, mode, meanmagoffset, pt
		if event.inaxes is not None: 
			if event.button==3: 
				xy = [event.xdata,event.ydata]
				
				#--------- finding object -----------
				if mode==1:
					dist = np.sqrt((xy[0]-(stats[0,:]+meanmagoffset)*flagplot)**2.+25.*(xy[1]-stats[1,:]*flagplot)**2.)
				elif mode==2:
					dist = np.sqrt((xy[0]-stats[2,:]*flagplot)**2.+(xy[1]-stats[1,:]*flagplot)**2.)
				index = np.argsort(dist)[0]

				#--------- printing object information ---------
				strinfo = ""
				strinfo += "------------------------------------------------------------------------\n"
				strinfo += "          object "+str(index+1)+"\n"
				strinfo += "------------------------------------------------------------------------\n"
				strinfo += "AsPyLib (vert,horiz) position : %(x).3f, %(y).3f" % {'x':obj[1,index],'y':obj[2,index]}+"\n"
				strinfo += "Prism (X,Y) position          : %(x).3f, %(y).3f" % {'x':1+obj[2,index],'y':1+obj[1,index]}+"\n"
				strinfo += "mean mag     : %.5f" % (stats[0,index]+meanmagoffset)+"\n"
				strinfo += "local noise  : %.5f" % stats[1,index]+"\n"
				strinfo += "global noise : %.5f" % stats[2,index]+"\n"
				strinfo += "quality flag : %.0f" % stats[3,index]+"\n"
				strinfo += "------------------------------------------------------------------------\n"
				if np.sum(np.shape(astrom))>0:
					strinfo += "measured position (alpha,delta) : " + astrom[0,index] +" "+ astrom[1,index]+"\n"
					strinfo += "------------------------------------------------------------------------\n"
					strinfo += "best match USNO-B1.0\n"
					strinfo += "- distance (arcsec)      : "+ usnob1[0,index]+"\n"
					strinfo += "- name                   : "+ usnob1[1,index]+"\n"
					strinfo += "- position (alpha,delta) : "+ usnob1[2,index]+" "+ usnob1[3,index]+"\n"
					strinfo += "- mag (B1,B2,R1,R2)      : "+ str(usnob1[4,index]) +" "+ str(usnob1[5,index]) +" "+ str(usnob1[6,index]) +" "+ str(usnob1[7,index])+"\n"
					strinfo += "------------------------------------------------------------------------"

				print
				print strinfo

				#--------- displaying star image ---------
				plt.sca(ax1)
				plt.cla()
				med = astro.get_median(image)[0]
				dsig = np.std(image.flatten())
				ax1.imshow(image, interpolation='nearest',cmap=cm.gray,origin='lower',vmin = med-0.5*dsig, vmax =med+2*dsig, aspect='equal', extent=None)
				ax1.set_xlim([obj[2,index]-50,obj[2,index]+50])
				ax1.set_ylim([obj[1,index]-50,obj[1,index]+50])
				ax1.scatter(obj[2,index],obj[1,index],s=200,marker='o',color='r',facecolors='none')
				plt.title('Image')
				
				#--------- plotting lightcurve and star image ---------
				plt.sca(ax2)
				plt.cla()
				plt.plot(data[:,0],data[:,index+1]+magoffset,label="object "+str(index+1))
				mask = ~(np.isnan(data[:,index+1]))
				mini = np.min(data[mask,index+1]+magoffset[mask])
				maxi = np.max(data[mask,index+1]+magoffset[mask])
				if maxi-mini<0.5:
					av = (maxi+mini)*0.5
					plt.ylim([av-0.25,av+0.25])
				plt.grid()
				plt.legend()
				plt.title('Lightcurve')
				limites = ax2.get_ylim()
				if limites[-1]>limites[0]:
					ax2.set_ylim(limites[::-1])
				plt.draw()
				
				#--------- adding mark on selected dot ---------
				if flag:
					pt.remove()
					if mode==1:
						ax.scatter(stats[0,index_old]+meanmagoffset,stats[1,index_old],s=80,marker='o',color='0.75',facecolors='none')
					elif mode==2:
						ax.scatter(stats[2,index_old],stats[1,index_old],s=80,marker='o',color='0.75',facecolors='none')
				if mode==1:
					pt = ax.scatter(stats[0,index]+meanmagoffset,stats[1,index],s=80,marker='o',color='b',facecolors='none')
				elif mode==2:
					pt = ax.scatter(stats[2,index],stats[1,index],s=80,marker='o',color='b',facecolors='none')
				flag = True
				index_old = index
		else:
			fig.canvas.mpl_disconnect(cid) 
			fig.canvas.stop_event_loop_default() 		

	cid = fig.canvas.mpl_connect('button_press_event', on_click) 
	fig.canvas.start_event_loop_default() 

warnings.resetwarnings() 

raw_input("finished")