# -*- coding: iso_8859_1 -*-

import numpy as np
from aspylib import conf
import os

def load_data(filename, string_start, string_end, rowstart=0, colstart=0, remove_blanks=False, text_display=True):
	"""
	---------------------
	Purpose
	This functions reads data (numerical or not) from a text file. It can read only part of the file. The data must be arranged in columns, separated by tabulations (\t).
	Blank lines (with only one character = '\n') are disregarded.
	The following sequences of characters are interpreted as numpy.nan values: "nan", "--", "1.#QNAN0" ; or as numpy.inf values: "inf", "+inf", "-inf".
	---------------------
	Inputs
	* filename (string) = complete filename (including path) of the text file that contains the data to be read.
	* string_start (string) = the beginning of the line where the function will start reading the file. This line is excluded from the data (default behavior) but can be included with the parameter "row_start", see below. If string_start is not found, no data is read.
	* string_end (string) = the beginning of the line where the function will stop reading the file. This line is excluded from the data. If string_end is not found, reading data is performed until the end of the file.
	* rowstart (integer) = optional parameter, default value is 0. Can take values equal to -1, 0 or positive integers. If equal to -1, the line that contains string_start is included in the data that is read. If equal to 0, nothing is added nor removed from the data : the first line that is read is the one just after the line that contains string_start. If equal to n>0, n lines are removed from the data.
	* colstart (integer) = optional parameter, default value is 0. The data that is read must be organized by columns, separated with tabulations (\t). colstart is the index of the first column that is read. If it is 0, all columns are read. If it is 1, the first column is excluded, etc.
	* remove_blanks (boolean) = optional argument. If equal to False (default), the empty strings ('') are kept in the data. If equal to True the empty strings are removed from the data.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output (numpy array of float or string) = the data is converted to a numpy array. A conversion to float numbers is attempted. If it fails, then the data is kept as string.
	---------------------
	"""

	#display file name
	if text_display:
		file_path = filename.split(conf.sep)
		file_core = file_path[len(file_path)-1]
		print
		print "... loading data in %s ..." % file_core

	#open file and reads data
	if os.path.exists(filename):
		f=open(filename, 'r')
		try:
			data = []
			flag = 0
			notlist = ["\n"]
			for line in f:
				#--- stops reading ---
				if flag:
					if line[:len(string_end)]==string_end:		
						break

				#--- starts reading (1st line included) ---
				if (1-flag):
					if line[:len(string_start)]==string_start:	
						flag = 1

				#--- reading data ---
				if flag:
					datalist = line.split('\t')
					if datalist[0][0] not in notlist:
						#stores data in array
						x = []
						for i,dd in enumerate(datalist):
							dd = dd.replace('\n','')
							if (i>=colstart):
								if (dd!='')or(not(remove_blanks)):
									if (dd[:8]=='1.#QNAN0')or(dd[:2]=='--')or(dd[:3]=='nan'):
										x.append(np.nan)
									elif (dd[:4]=='+inf')or(dd[:4]=='-inf')or(dd[:3]=='inf'):
										x.append(np.inf)
									else:
										x.append(dd)
						data.append(x)

		finally:
			f.close()
		data = data[rowstart+1:]
		try:
			return np.array(data,dtype=np.float64)
		except:
			return np.array(data)
	else:
		raise Exception("aspylib.astro.load_data() error: file not found" )
		