# -*- coding: iso_8859_1 -*-

from aspylib import astro, conf
import numpy as np
import calib7

def calibrate(Imagelist, Darklist, Flat):

	#------ creating dark frame ------
	darks = astro.get_imagedata(Darklist)
	dark_final = astro.add_median(darks)
	hotpixels = np.tile(calib7.find_hotpixels(dark_final, 100),(1,1,1))

	#------ creating flat frame ------
	flat = astro.get_imagedata([Flat])
	flat = flat * 1.2/ astro.get_median(flat)
	flat = flat + 1e-10

	#------ images calibration ------
	images = astro.get_imagedata(Imagelist)

	images_calib = (images - dark_final) / flat
	images_avg = (np.roll(images_calib,1,axis=1) + np.roll(images_calib,-1,axis=1) + np.roll(images_calib,1,axis=2) + np.roll(images_calib,-1,axis=2)) / 4.
	images_calib = images_avg*hotpixels + images_calib*(1.-hotpixels)

	#checking for saturation
	test = (images_calib<conf.saturation-0.001)*(images!=conf.saturation)
	images_calib = (1.-test)*conf.saturation + test*images_calib

	return images_calib


def calibrate_and_save(Imagelist, Darklist, Flat, Imagelist2, Config, Headers={}):

	#------ creating dark frame ------
	darks = astro.get_imagedata(Darklist)
	dark_final = astro.add_median(darks)
	hotpixels = calib7.find_hotpixels(dark_final, 100)

	#------ creating flat frame ------
	flat = astro.get_imagedata([Flat])
	flat = flat * 1.2 / astro.get_median(flat)
	flat = flat + 1e-10

	#------ images calibration and saving ------
	for i in range(len(Imagelist)):
		#loading
		imagei = astro.get_imagedata([Imagelist[i]])

		#calibration
		imagei_calib = (imagei - dark_final) / flat
		imagei_avg = (np.roll(imagei_calib,1,axis=0) + np.roll(imagei_calib,-1,axis=0) + np.roll(imagei_calib,1,axis=1) + np.roll(imagei_calib,-1,axis=1)) / 4.
		imagei_calib = imagei_avg*hotpixels + imagei_calib*(1.-hotpixels)

		#checking for saturation
		test = (imagei_calib<conf.saturation-0.001)*(imagei!=conf.saturation)
		imagei_calib = (1.-test)*conf.saturation + test*imagei_calib
	
		print 'dark and flat calibration ... done'

		#save image
		headerdict = astro.create_headers(Config, [Imagelist[i]])[0]
		headerdict.update(Headers)
		astro.save_imagelist(imagei_calib, [headerdict], [Imagelist2[i]])
		print Imagelist2[i] + ' saved'
	#-------------------------------------
