#include "dll.h"

extern "C"
{
   __declspec(dllexport) int matching_pairs(double dist, int dim1, double* x1, double* y1, int dim2, double* x2, double* y2, int& dim, int* z1, int*z2)
   {
       /*--------------------------------------------------------------------------------
       //   This function calculates the matching pairs between two lists of stars
       //   - first list has dim1 stars with coordinates (x1,y1)
       //   - second list has dim2 stars with coordinates (x2,y2)\
       //   a pair of stars is matching if abs(x2-x1)+abs(y2-y1) < dist
       //--------------------------------------------------------------------------------*/

        //**********************************************************************************************
        //  for each (x1,y1) find the closest (x2,y2) that is below threshold distance
        //**********************************************************************************************

        double dd, dd2;

        //--- no matching pair at start ---
        dim = 0;

        //--- start with 0 so that it is re-initialised at the end of the loop only if needed
        //and not at the beginning of all loops ---
        int nb=0;

        for (int i=0; i<dim1; i++) {
        for (int j=0; j<dim2; j++) {
            //--- cascaded tests ---
            if (x2[j]-x1[i]<dist)
            if (x2[j]-x1[i]>-dist)
            if (y2[j]-y1[i]<dist)
            if (y2[j]-y1[i]>-dist)
            {
                dd2 = fabs(x2[j]-x1[i]) + fabs(y2[j]-y1[i]);

                if (nb>0)   //then a matching star j is already known for star i
                {
                    if (dd2 <dd)
                    {
                        dd = dd2;
                        z1[dim] = i;
                        z2[dim] = j;
                    }
                }
                else
                {
                    dd = dd2;
                    z1[dim] = i;
                    z2[dim] = j;
                    nb++;
                }
            }
        }
        if (nb>0)
        {
            dim++;
            nb=0;
        }
        }

        //**********************************************************************************************
        //  identify duplicates, for instance if the same z2 corresponds to two different z1
        //**********************************************************************************************

        bool flag;
        int nbremoved=0,*removed;
        removed = (int *)malloc((size_t)(dim * (long)sizeof(int)));

        for (int i=0; i<dim-1; i++) {
        for (int j=i+1; j<dim; j++) {

            if (z2[i]==z2[j])
            {
                //********** is one of the two indices already removed ? **************
                flag = true;
                for (int k=0; k<nbremoved; k++)
                {
                    if (i==removed[k])
                    {
                        flag = false;
                        break;
                    }
                    if (j==removed[k])
                    {
                        flag = false;
                        break;
                    }
                }
                //********** if not, then we need to compare the distances to decide which index to remove **************
                if (flag)
                {
                    dd = fabs(x2[z2[i]]-x1[z1[i]]) + fabs(y2[z2[i]]-y1[z1[i]]);
                    dd2 = fabs(x2[z2[j]]-x1[z1[j]]) + fabs(y2[z2[j]]-y1[z1[j]]);

                    if (dd<=dd2)
                    {
                        removed[nbremoved]=j;
                        nbremoved++;
                    }
                    else
                    {
                        removed[nbremoved]=i;
                        nbremoved++;
                        break;      //then immediatly switch to next i
                    }
                }
            }
        }}

        //**********************************************************************************************
        //  removing duplicates
        //**********************************************************************************************

        int offset=0;

        for (int i=0; i<=dim; i++)
        {
            flag = true;
            for (int k=0; k<nbremoved; k++)
                if (i==removed[k])
                {
                    flag = false;
                    break;
                }
            if (flag)
            {
                z1[i-offset] = z1[i];
                z2[i-offset] = z2[i];
            }
            else
            {
                offset++;
            }
        }
        dim = dim - offset;

        free(removed);

      return(0);
   }
}
