# -*- coding: iso_8859_1 -*-

from aspylib import astro, conf
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.font_manager as fm
import os

def filter_magnitudes(obj_index, names, sizes, mag, rejdata):

	nb_ref = len([i for i in names if (i[0]=='r')])
	nb_mov = len([i for i in names if (i[0]=='m')])
	nb_vbl = len([i for i in names if (i[0]=='v')])
	nb_obj = nb_mov + nb_vbl

	sx, sy = astro.get_window_size(conf.cmd)
	
	#------------------------------------------------------------------------------------	
	#		initialisations
	#------------------------------------------------------------------------------------	

	mag_i = mag.copy()		#copy of masked array "mag"
	
	rejdata_i = list(rejdata)
	rejdisk_i = []

	#------------------------------------------------------------------------------------	
	#		first check to remove wrong measurements affecting only the targets
	#		(not ref stars, for which wrong data have been already removed)
	#------------------------------------------------------------------------------------	

	print
	print '------ REMOVING WRONG MEASUREMENTS (IF ANY) -------------------------'
	print

	for i in range(len(mag_i)):
		if i not in rejdata:
			test_image = np.sum(np.isnan(mag_i[i,obj_index,:]).flatten())
			if test_image > 0:
				print "Image " + str(i+1) + " has wrong measurements, it is removed"
				rejdata_i.append(i)
				mag_i.mask[i,:,:] = True
			
	#------------------------------------------------------------------------------------	
	#		second check based on visual inspection
	#------------------------------------------------------------------------------------	
	
	fig = plt.figure()
	fig.canvas.set_window_title(names[obj_index]) 
	plt.axes([0.08,0.05,0.7,0.85])

	indices = ~(mag_i.mask[:,0,0])	#indexing unmasked images
	for j in range(len(sizes)):		
		mag_plot = mag_i[indices,obj_index,j] - np.mean(mag_i[:,nb_obj:,j], axis=1)[indices]
		plt.plot(mag_plot - np.mean(mag_plot), conf.col2[j%16], label=str(sizes[j]))
		
	plt.legend(prop=fm.FontProperties(size=11), bbox_to_anchor=(1.02, 1.), loc=2)	
	plt.title(names[obj_index])
	plt.grid()
	ax = plt.gca()
	ax.set_ylim(ax.get_ylim()[::-1])
	fx, fy = astro.get_window_position(names[obj_index])
	astro.move_window(names[obj_index], [sx, fy])
	plt.draw()

	if (obj_index<nb_mov):
		print
		print '------ CHECKING DIFF MAGNITUDES FOR MOVING OBJECT',obj_index+1,'-----------------'
		print
		print 'You have now the possibility to reject some data. For instance, data corresponding to the asteroid crossing a star is detected because it gives smaller magnitudes (larger fluxes) for larger disk sizes.'
	elif (obj_index>=nb_mov):
		print
		print '------ CHECKING DIFF MAGNITUDES FOR VARIABLE STAR',obj_index+1-nb_mov,'-----------------'
		print
		print 'You have now the possibility to reject some data.'

	ans = raw_input('Do you want to do it (yes/no) ?')
	if ans=='yes':
		#--------------------------------------------------------------------------------------------------
		if (len(sizes)>1):
			print
			print 'Rejection of photometry disks...'
			print 'if you want to reject the k-th specified disk, type k (for 1st disk write 1, 2nd disk write 2, etc)'
			print 'when finished, press Enter'
			while 1:
				try:
					val = int(raw_input('disk to be rejected:'))
					rejdisk_i.append(val-1)
					mag_i.mask[:,:,val-1] = True
				except:
					break
		#--------------------------------------------------------------------------------------------------
		print
		print 'Rejection of photometric data...'
		while 1:
			try:
				ans = int(raw_input('how many intervals do you want to reject ?'))
				break
			except:
				print "please enter an integer !"
		for j in range(ans):
			print
			print 'Interval',j+1
			while 1:
				try:
					mini = int(raw_input('first rejected data:'))
					break
				except:
					print "please enter an integer !"
			while 1:
				try:
					maxi = int(raw_input('last rejected data:'))
					break
				except:
					print "please enter an integer !"
			interval = range(min([mini,maxi]),max([mini, maxi])+1)
			rejdata_i = rejdata_i + list(np.where(indices)[0][interval])
			mag_i.mask[list(np.where(indices)[0][interval]),:,:] = True
		#---------------------------------------------------------------------
	plt.close()
	print
	
	return mag_i, rejdata_i, rejdisk_i