# -*- coding: iso_8859_1 -*-

from aspylib import astro, conf
import matplotlib.pyplot as plt
import numpy as np
import os, datetime, time, pp

def calculate_shifts(Imagelist, outputname, detection_area = 10, nb_stars = 12):
	t0 = time.time()

	#------ setting up console windows ------
	astro.maximize_window(conf.cmd)
	sx, sy = astro.get_window_size(conf.cmd)

	#------ loading image + initialising shifts ------
	shifts = [[0.,0.] for i in range(len(Imagelist))]
	image1 = astro.get_imagedata([Imagelist[0]], text_display=False)
	calculation_list = range(1,len(Imagelist))
	
	#------ initialising image area selection parameters ------
	excluded = []
	x0 = 0
	y0 = 0
	x1, y1 = np.shape(image1)
	
	#************************************************************************************
	while True:		
		fig, ax = astro.display(image1, title="First image")
		fx, fy = astro.get_window_position("First image")
		astro.move_window("First image", [sx, fy])
		astro.set_focus(conf.cmd)

		print
		print 'Image shift calculation can be perturbated by slowly moving objects, strongly saturated stars or artefacts. If the "star matching" method is used (default), diffuse objects may also be problematic.'
		print
		print 'There are 3 possible approaches to calculate the shifts:'
		print '(1) perform star detections over the complete images'
		print '(2) exclude some image areas for the star detection'
		print '(3) detect only the stars in one subpart of the image'
		print
		while 1:
			try:
				ans = int(raw_input('What option do you want to use (1/2/3) ?'))
				break
			except:
				print "please enter an integer !"
		if ans==2:
			while 1:
				try:
					ans = int(raw_input('How many areas do you want to exclude ?'))
					break
				except:
					print "please enter an integer !"
			for i in range(ans):
				print 'Select 4 corners of area',i+1,'to be excluded'
				xy = astro.select(fig,ax,4)

				excluded.append([min([xy[0][0],xy[1][0],xy[2][0],xy[3][0]]), \
								max([xy[0][0],xy[1][0],xy[2][0],xy[3][0]]), \
								min([xy[0][1],xy[1][1],xy[2][1],xy[3][1]]), \
								max([xy[0][1],xy[1][1],xy[2][1],xy[3][1]])])
		elif ans==3:
			print 'Select 4 corners of area to be used'
			xy = astro.select(fig,ax,4)

			x0 = min([xy[0][0],xy[1][0],xy[2][0],xy[3][0]])
			x1 = max([xy[0][0],xy[1][0],xy[2][0],xy[3][0]])
			y0 = min([xy[0][1],xy[1][1],xy[2][1],xy[3][1]])
			y1 = max([xy[0][1],xy[1][1],xy[2][1],xy[3][1]])

		plt.close()

		#------------------------------------------------------------------------------------
		print
		stars1 = astro.find_stars_accurate(image1[x0:x1,y0:y1], nb_stars, detection_area, excluded=excluded, text_display = False)
		
		print 'shifts of image :',Imagelist[0].split(conf.sep)[-1]
		print "dx:",0.
		print "dy:",0.
		#shifts[0] = [0.,0.]		#not needed (shifts is initialised with [0.,0.] for all images at beginning of the function)
		
		def parallel_imageshift(i, Imagelist, x0, x1, y0, y1, nb_stars, detection_area, excluded, stars1):
			print 'calculating shifts of image :',Imagelist[i].split(aspylib.conf.sep)[-1]
			image = aspylib.astro.get_imagedata([Imagelist[i]], text_display=False)
			stars = aspylib.astro.find_stars_accurate(image[x0:x1,y0:y1], nb_stars, detection_area, excluded=excluded, text_display = False)
			xs, ys = aspylib.astro.find_shift_stars_matching(stars1, stars, text_display = False)
			print "dx:",xs
			print "dy:",ys
			return [xs,ys]

		# tuple of all parallel python servers to connect with
		job_server = pp.Server()
		print "Starting pp with", job_server.get_ncpus(), "workers"
		
		jobs = []
		for i in calculation_list:	
			jobs.append(job_server.submit(parallel_imageshift,(i, Imagelist, x0, x1, y0, y1, nb_stars, detection_area, excluded, stars1,), (), ("aspylib.astro", "aspylib.conf",)))
			
		for i in range(len(jobs)):
			shifts[calculation_list[i]] = jobs[i]()

		job_server.destroy()
		
		#for i in calculation_list:	
		#	shifts[i] = parallel_imageshift(i, Imagelist, x0, x1, y0, y1, nb_stars, detection_area, excluded, stars1)
			
		print time.time()-t0,"seconds"

		#------------------------------------------------------------------------------------
		fig = plt.figure()
		fig.canvas.set_window_title("Image shifts") 
		fx, fy = astro.get_window_position("Image shifts")
		astro.move_window("Image shifts", [sx, fy])
		plt.plot(np.array(shifts)[:,0], 'b')
		plt.plot(np.array(shifts)[:,1], 'g')
		plt.grid()
		plt.title('Image shifts: Vertical (blue), Horizontal (green)')
		plt.xlabel('Image number')
		plt.ylabel('Image shifts (pixels)')
		plt.draw()	
		astro.set_focus(conf.cmd)

		print		
		print 'You must now choose one of the 4 possibilities:'
		print '(1) you accept all the calculated shifts'
		print '(2) you accept the calculated shifts but you want to remove a few points'
		print '(3) you accept only part of the calculated shifts, you want to recalculate the shifts for a few points'
		print '(4) you are not happy with the results, you want to restart all the calculation'
		print
		while 1:
			try:
				choice = int(raw_input('What is your decision (1/2/3/4) ?'))
				break
			except:
				print "please enter an integer !"

		if (choice == 1)or(choice == 2):	#the shifts are accepted, but maybe a few points will be removed
			break
		elif choice == 3:					#the shifts are partly accepted, and a few points will be re-calculated	
			calculation_list = []
			print
			print 'You now need to specify for which images you want to recalculate the shifts.'
			while 1:
				try:
					im = int(raw_input("Please enter the numbers of the images for shift re-calculation:"))
					calculation_list.append(im)
				except:
					break
			calculation_list = list(set(calculation_list))
			calculation_list.sort()
			plt.close()
		else:								#the shifts are rejected and the calculation is re-started from the beginning
			plt.close()
			shifts = [[0.,0.] for i in range(len(Imagelist))]
			calculation_list = range(1,len(Imagelist))
		
		
	rej = []
	if (choice==1):		#all calculated shifts are accepted
		plt.close()
	elif (choice==2):	#the calculated shifts are accepted but a few points will be removed
		print
		print
		print '--- IMAGES WITH INCORRECT SHIFT CALCULATION -------------------------'
		print
		print 'You have now the possibility to reject some images for which the shift was not correctly calculated.'
		while 1:
			try:
				im = int(raw_input("Please enter the numbers of the images to be removed:"))
				rej.append(im)
			except:
				break
		rej = list(set(rej))
		rej.sort()
		plt.close()

	#------------------------------------------------------------------------------------
	f = open(outputname, 'w')
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write("Working directory:\n"+(os.path.split(Imagelist[0])[0]).encode('iso_8859_1')+"\n\n")
	f.write("-------------------------------------------------\n")
	f.write("      User inputs\n")
	f.write("-------------------------------------------------\n")
	f.write("Images rejected by user:\t")
	if len(rej)==0:
		f.write('none\n')
	else:
		for i in range(len(rej)-1):
			f.write(str(rej[i]+1)+',')
		f.write(str(rej[-1]+1)+"\n")
	f.write("\n")
	
	f.write("-------------------------------------------------\n")
	f.write("      Images shifts are\n")
	f.write("-------------------------------------------------\n")
	f.write('Image\tImage status\tImage name\tX shift (vertical)\tY shift (horizontal)\n')
	for i,im in enumerate(Imagelist):
		f.write(str(i+1)+'\t')
		if i not in rej:
			f.write("ok\t")
		else:
			f.write("rejected\t")
		f.write(im.split(conf.sep)[-1]+'\t'+"%.4f"%shifts[i][0]+'\t'+"%.4f"%shifts[i][1]+'\n')
	f.close()
	
	#------------------------------------------------------------------------------------
	Imagelist2 = [Imagelist[i] for i in range(len(Imagelist)) if i not in rej]
	shifts2 = np.array([shifts[i] for i in range(len(shifts)) if i not in rej])

	return Imagelist2, shifts2
