# -*- coding: iso_8859_1 -*-

from aspylib import astro, conf
import numpy as np
import os, datetime

def make_photometrystats(Imagelist, sizes, coord, CCD, method='centroid', check_sat = [True,'saturation'], mag_sat = [np.nan, np.nan], mag_zero = [np.nan, np.nan], error_messages = 1):

	#loop on images
	print
	mag_total = []
	SNR_total = []
	
	for i in range(len(Imagelist)):
		#loads relevant image
		print 'Photometry on :',Imagelist[i].split(conf.sep)[-1]
		image = astro.get_imagedata([Imagelist[i]], text_display=False)

		x,y = aperture_photometrystats(image, sizes, coord[i], CCD, method=method, check_sat = check_sat, mag_sat = mag_sat, mag_zero = mag_zero, error_messages = error_messages)
		mag_total.append(x)
		SNR_total.append(y)
		
	return np.array(mag_total), np.array(SNR_total)

def aperture_photometrystats(image, sizes, coord, CCD, method='centroid', check_sat = [True,'saturation'], mag_sat = [np.nan, np.nan], mag_zero = [np.nan, np.nan], error_messages = 1):

	if check_sat[1]=='saturation':
		check_sat[1] = conf.saturation
	
	maxsize = int(np.ceil(max(sizes)[0]))
	#-------------------------
	if method == 'centroid':
		image2 = np.array(image)
	#-------------------------
	
	error_image_border = []
	error_negative_intensity = []
		
	#loop on stars / asteroids
	mag_stars = []
	SNR_stars = []

	for j, xy in enumerate(coord):

		mag_methods = []
		SNR_methods = []

		if np.isnan(xy[0]):
			for k in range(len(sizes)):		
				mag_methods.append(np.nan)
				SNR_methods.append(np.nan)
		else:
			#coordinates for object center
			x = int(np.round(xy[0]))
			y = int(np.round(xy[1]))
			dx = xy[0] - x
			dy = xy[1] - y
		
			#-------------------------		
			saturation = False
			if check_sat[0]:
				xnot = max(0,x-maxsize)
				xrange = min(image.shape[0],x+maxsize+1)
				ynot = max(0,y-maxsize)
				yrange = min(image.shape[1],y+maxsize+1)
				try:
					maxsignal = np.max(image[xnot:xrange,ynot:yrange].flatten())
					if maxsignal>=check_sat[1]:
						saturation = True
				except:	#if image[xnot:xrange,ynot:yrange]=[] because point outside image
					pass
			#-------------------------
						
			if saturation:
				#loop on apertures for photometry
				for k in range(len(sizes)):		
					mag_methods.append(mag_sat[0])
					SNR_methods.append(mag_sat[1])
			else:
				#-------------------------
				if method == 'centroid':
					xnotmax = max(0,x-maxsize-3)
					xrangemax = min(image.shape[0],x+maxsize+4)
					ynotmax = max(0,y-maxsize-3)
					yrangemax = min(image.shape[1],y+maxsize+4)
					if (xrangemax>xnotmax)and(yrangemax>ynotmax):
						image2[xnotmax:xrangemax,ynotmax:yrangemax] = astro.translate_Bspline(image2[xnotmax:xrangemax,ynotmax:yrangemax],(-dx,-dy),3)
				#-------------------------

				#loop on apertures for photometry
				for k in range(len(sizes)):		

					#-------------------------
					if method == 'centroid':
						test = (x-sizes[k][0]-3<0)or(x+sizes[k][0]+4>image.shape[0])or(y-sizes[k][0]-3<0)or(y+sizes[k][0]+4>image.shape[1])
					else:
						test = (x-sizes[k][0]<0)or(x+sizes[k][0]+1>image.shape[0])or(y-sizes[k][0]<0)or(y+sizes[k][0]+1>image.shape[1])
					#-------------------------

					if 	test:
						error_image_border.append([j+1,k+1])
						mag_methods.append(np.nan)
						SNR_methods.append(np.nan)
					else:
						#selecting subimage
						xnot = max(0,x-int(sizes[k][2]))
						xrange = min(image.shape[0],x+int(sizes[k][2]+1))
						ynot = max(0,y-int(sizes[k][2]))
						yrange = min(image.shape[1],y+int(sizes[k][2]+1))

						subimage = image[xnot:xrange,ynot:yrange]

						#aperture photometry
						grid = np.indices(subimage.shape)
						radius = (grid[0]-x+xnot)**2+(grid[1]-y+ynot)**2

						cond1 = radius<=sizes[k][0]**2
						cond2 = (radius>=sizes[k][1]**2)*(radius<=sizes[k][2]**2)

						#-------------------------
						if method == 'centroid':
							subimage2 = image2[xnot:xrange,ynot:yrange]
							center = np.extract(cond1,subimage2)
						else:
							center = np.extract(cond1,subimage)
						#-------------------------
						crown = np.extract(cond2,subimage)

						#print np.where(radius==np.min(np.min(radius)))
						#print np.where(subimage==np.max(np.max(subimage)))
						#raw_input("ok?")

						#------ magnitude calculation ------
						floor = astro.get_median(np.tile(np.array(crown,dtype=np.float),(1,1,1)))[0]
						intensity = np.sum(np.array(center,dtype=np.float)-floor)
						if intensity <= 0.:
							error_negative_intensity.append([j+1,k+1])
							mag_methods.append(mag_zero[0])
							SNR_methods.append(mag_zero[1])
						else:
							mag_methods.append(-2.5*np.log10(intensity))
							nelec_star = intensity * CCD['gain'] / CCD['flatgain']		#total nb of electrons coming from star
							nelec_sky = floor * CCD['gain'] / CCD['flatgain']			#nb of sky electrons per pixel
							npix_star = np.product(np.shape(center))
							npix_sky = np.product(np.shape(crown))
							SNR_methods.append(nelec_star / np.sqrt(nelec_star + npix_star*(1.0 + npix_star/npix_sky)*(nelec_sky + CCD['ROnoise']**2. + (CCD['gain']**2.)/12.)))
	
				#-------------------------
				if method == 'centroid':
					xnotmax = max(0,x-maxsize-3)
					xrangemax = min(image.shape[0],x+maxsize+4)
					ynotmax = max(0,y-maxsize-3)
					yrangemax = min(image.shape[1],y+maxsize+4)
					if (xrangemax>xnotmax)and(yrangemax>ynotmax):
						image2[xnotmax:xrangemax,ynotmax:yrangemax] = image[xnotmax:xrangemax,ynotmax:yrangemax]
				#-------------------------
		mag_stars.append(mag_methods)
		SNR_stars.append(SNR_methods)
	
	if error_messages==1:
		if len(error_image_border)>0:
			print 'photometry.aperture_photometry():',len(error_image_border),'/',len(coord)*len(sizes),'measurement(s) on image border'
		if len(error_negative_intensity)>0:
			print 'photometry.aperture_photometry():',len(error_negative_intensity),'/',len(coord)*len(sizes),'measurement(s) with <0 intensity'

	elif error_messages==2:
		if len(error_image_border)>0:
			print 'photometry.aperture_photometry():',len(error_image_border),'/',len(coord)*len(sizes),'measurement(s) on image border:'
			print str(error_image_border)[1:-1]
		if len(error_negative_intensity)>0:
			print 'photometry.aperture_photometry():',len(error_negative_intensity),'/',len(coord)*len(sizes),'measurement(s) with <0 intensity:'
			print str(error_negative_intensity)[1:-1]

	#-------------------------
	if method == 'centroid':
		del image2
	#-------------------------
	return np.array(mag_stars), np.array(SNR_stars)

def write_magnitudes_and_SNR(Imagelist, outputname, names, xy, dateobs, sizes, mag, SNR):
	
	if os.path.exists(outputname):
		os.remove(outputname)

	f = open(outputname, 'w')

	#--- file header ---
	f.write('date = '+str((datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S"))+'\n')
	f.write("Working directory:\n"+(os.path.split(Imagelist[0])[0]).encode('iso_8859_1')+"\n\n")

	f.write('datation at start of exposure\n')
	h1 = astro.get_headervalues([Imagelist[0]],'EXPOSURE')[0]
	h2 = astro.get_headervalues([Imagelist[0]],'EXPTIME')[0]
	if h1!='':
		f.write('exposure time (sec):\t'+str(int(np.round(h1)))+'\n\n')
	elif h2!='':
		f.write('exposure time (sec):\t'+str(int(np.round(h2)))+'\n\n')

	#--- stars X,Y coordinates ---
	f.write('objects coordinates on first frame\n')
	f.write('object')
	for j in range(len(xy[0])):
		f.write('\t'+names[j])
	f.write('\n')
	f.write('X position (vertical)')
	for j in range(len(xy[0])):
		f.write('\t'+"%.4f"%xy[0][j][0])
	f.write('\n')
	f.write('Y position (horizontal)')
	for j in range(len(xy[0])):
		f.write('\t'+"%.4f"%xy[0][j][1])
	f.write('\n\n')

	#--- measured magnitudes ---
	for k in range(len(sizes)):
		f.write('magnitudes from apertures: '+str(sizes[k][0])+' - '+str(sizes[k][1])+' - '+str(sizes[k][2])+'\n')

		#magnitudes
		f.write('Image\tImage name\tJulian day')
		for j in range(len(xy[0])):
			f.write('\t'+names[j])
		f.write('\n')
		for i,im in enumerate(Imagelist):
			f.write(str(i+1)+'\t')
			f.write(im.split(conf.sep)[-1]+'\t')
			f.write(('%.6f' % dateobs[i]))
			for j in range(len(xy[i])):
				f.write('\t'+'%.6f' % mag[i,j,k])
			f.write('\n')
		f.write('\n')

	#--- measured SNRs ---
	for k in range(len(sizes)):
		f.write('SNRs from apertures: '+str(sizes[k][0])+' - '+str(sizes[k][1])+' - '+str(sizes[k][2])+'\n')

		#magnitudes
		f.write('Image\tImage name\tJulian day')
		for j in range(len(xy[0])):
			f.write('\t'+names[j])
		f.write('\n')
		for i,im in enumerate(Imagelist):
			f.write(str(i+1)+'\t')
			f.write(im.split(conf.sep)[-1]+'\t')
			f.write(('%.6f' % dateobs[i]))
			for j in range(len(xy[i])):
				f.write('\t'+'%.2f' % SNR[i,j,k])
			f.write('\n')
		f.write('\n')

	f.close()
	#-------------------------------------